<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Knp\Snappy\Pdf;
use Symfony\Component\HttpFoundation\Response;

use App\Entity\BranchSmsEntity;
use App\Entity\BranchEntity;
use App\Entity\SmsEntity;


use App\Service\AuthService;

/**
 * @Route("/api")
 */
class ApiController extends AbstractController
{

    /**
     * @Route("/get_for_sent_sms", name="api_get_for_sent_sms")
     */
    public function get_for_sent_sms(Request $request, AuthService $authService)
    {

       $result = [ ];
       $em = $this->getDoctrine()->getManager();
       $bs = $em->getRepository(BranchSmsEntity::class)->findOneBy([
            'status' => [
                'New',
                'Sending'
            ]
       ]);
      
       if($bs){

            if(!$this->validatePhoneNo($bs->getContactNo())){
               
                $bs->setStatus('Invalid Contact No.');
                $em->flush();
            } else {
               
                $result = [ 
                    'id' => $bs->getId(),
                    'contact_no' => $bs->getContactNo(),
                    'message' => $bs->getMessage(),
                    'sent_ctr' => strval($bs->getSentCtr() ? $bs->getSentCtr() : 0) 
                ];
            }
        }
       
       return new JsonResponse($result);
    }

    /**
     * @Route("/update_sms/{id}/{status}/{sent_ctr}", name="api_update_sms")
     */
    public function update_sms(Request $request, AuthService $authService, $id, $status, $sent_ctr)
    {

       $result = [ 'success' => true, 'msg' => ''];
       $em = $this->getDoctrine()->getManager();
       $bs = $em->getRepository(BranchSmsEntity::class)->find($id);
      
       if($bs){

            if($status == 'Sending' && $sent_ctr >= 5){
                $status = 'Invalid Contact No';
            }

            $bs->setSentCtr($sent_ctr);
            $bs->setStatus($status);
            $bs->setSendAt(new \DateTime( date('Y-m-d H:i:s') ));
            $em->flush();
       }
       
       return new JsonResponse($result);
    }

    private function validatePhoneNo($phoneNo){
        
        if(preg_match("/^(09|\+639)\d{9}$/", $phoneNo)) {
           return true;
        }
        
        return false;
        
    }
    
}
