<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;

use App\Service\AuthService;
use App\Entity\ClientEntity;
use App\Entity\BreedEntity;
use App\Entity\SpeciesEntity;
use App\Entity\BranchEntity;
use App\Entity\ClientPetEntity;




use App\Form\ClientForm;

/**
 * @Route("/client")
 */
class ClientController extends AbstractController
{
    /**
     * @Route("/ajax_form", name="client_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {

       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $client = $em->getRepository(ClientEntity::class)->find(base64_decode($formData['id']));
       
       $formChoices = [
         'breeds' => $em->getRepository(BreedEntity::class)->getAllInArray($authService),
         'species' => $em->getRepository(SpeciesEntity::class)->getAllInArray($authService)
       ];

 
       if(!$client) {
          $client = new ClientEntity();
       }
       $formOptions = array('action' => $formData['action'],  'branchId' => $authService->getUser()->getBranch()->getId());
       $form = $this->createForm(ClientForm::class, $client, $formOptions);
    
       $result['html'] = $this->renderView('Client/ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Client',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/client/ajax_form.js') ,
            'formChoices' => json_encode($formChoices),
            'client' => $client,
            'formChoicesInarray' => $formChoices
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="client_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];


       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){

         $em = $this->getDoctrine()->getManager();

         $formChoices = [
            'breeds' => $em->getRepository(BreedEntity::class)->getAllInArray($authService),
            'species' => $em->getRepository(SpeciesEntity::class)->getAllInArray($authService)
          ];

         $clientForm = $request->request->get('client_form');
         
         $errors = $em->getRepository(ClientEntity::class)->validate($clientForm);

         if(!count($errors)){
            
            $client = $em->getRepository(ClientEntity::class)->find($clientForm['id']);
            
            if(!$client) {
               $client = new ClientEntity();
            }
     
            $formOptions = array('action' => $clientForm['action'], 'branchId' => $authService->getUser()->getBranch()->getId());
            $form = $this->createForm(ClientForm::class, $client, $formOptions);


            switch($clientForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($client);
                        $em->flush();

                        $this->processPet($clientForm, $client, $em);
                        $result['msg'] = 'Record successfully added to record.';

                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);
                     if ($form->isValid()) {
                        
                        $em->persist($client);
                        $em->flush();

                        $this->processRemove($clientForm, $client, $em);
                        $this->processPet($clientForm, $client, $em);

                        $result['msg'] = 'Record successfully updated.';
                     } else {
                       
                        $result['error'] = 'Ooops something went wrong please try again later.';

                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $client->setIsDeleted(true);
                        $em->flush();
                        
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

   /**
    * @Route("/ajax_delete", name="client_ajax_delete")
    */
   public function ajaxDelete(Request $request, AuthService $authService): JsonResponse
   {

      $result = [ 'success' =>  true, 'msg' => ''];

      if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
      
      if($request->getMethod() == "POST"){
        
         $id = $request->request->get('id');
         $em  = $this->getDoctrine()->getManager();
         $client = $em->getRepository(ClientEntity::class)->find(base64_decode($id));

         if(!is_null($client)){

            $client->setIsDeleted(true);
            $em->flush();

            $result['msg'] = 'Client successfully deleted';
 
         } else {

            $result['success'] = false;
            $result['msg'] = 'Something went wrong please try again.';

         }
      }

      return new JsonResponse($result);

   }

      /**
    * @Route("", name="client_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Client'))) return $authService->redirectToHome();
      
      $page_title = ' Owner'; 
      return $this->render('Client/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/client/index.js') ]
      );
   }



   /**
    * @Route("/autocomplete", name="client_autocomplete")
    */
   public function autocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'clients',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(ClientEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }


   /**
    * @Route("/ajax_list", name="client_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(ClientEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

   private function processOrganizations($form, $client, $em){

      if(isset($form['organization'])){
         foreach ($form['organization'] as $key => $i) {

            $clientOrg = $em->getRepository(ClientOrganizationEntity::class)->findOneBy(array(
               'client' => $client->getId(),
               'organization' => $i,
               'isDeleted' => 0
            ));

            if(!$clientOrg){
               $clientOrg =  new ClientOrganizationEntity();
               $clientOrg->setOrganization($em->getReference(OrganizationEntity::class, $i));
               $clientOrg->setClient($client);
            }

            $em->persist($clientOrg);
            $em->flush();
         }
         
      }
   }

   private function processPet($form, $client, $em){


      if(isset($form['pet'])){
         
         foreach ($form['pet'] as $key => $i) {

            if(isset($i['client_pet_id'])){
               $clientPet = $em->getRepository(ClientPetEntity::class)->find(base64_decode($i['client_pet_id']));
            } else {

               $clientPet =  new ClientPetEntity();
            }   

            $clientPet->setPetName(isset($i['name']) ? $i['name'] : '');
            $clientPet->setSpecies(isset($i['species']) ? $em->getReference(SpeciesEntity::class, $i['species']) : NULL);
            $clientPet->setBreed(isset($i['breed']) ?  $em->getReference(BreedEntity::class, $i['breed'])  : NULL);
            $clientPet->setRemarks(isset($i['remarks']) ?  $i['remarks']   :  '');
            $clientPet->setClient($client);
            $em->persist($clientPet);
            $em->flush();

            $this->processFile($key, $clientPet, $em);
         }
         
      }
   }

   private function processFile($k , $clientPet, $em){
      if(isset($_FILES) && isset($_FILES['client_form']) && isset($_FILES['client_form']['name']) && isset($_FILES['client_form']['name']['pet']) && isset($_FILES['client_form']['name']['pet'][$k])){
       
         $baseName = $clientPet->getId() . '-' . time() . '.' . pathinfo($_FILES['client_form']['name']['pet'][$k]['file'], PATHINFO_EXTENSION);
         $uploadFile = $clientPet->getUploadRootDir() . '/' . $baseName;

         if(move_uploaded_file($_FILES['client_form']['tmp_name']['pet'][$k]['file'], $uploadFile)) {
            $clientPet->removeFile();
            $clientPet->setFileDescription($_FILES['client_form']['name']['pet'][$k]['file']);
            $clientPet->setParsedFileDescription($baseName);
         }

         $em->flush();
      }  

      
   }

   private function processRemove($form, $client, $em){

    
      $petIds = isset($form['client_pet_ids']) ? $form['client_pet_ids'] : []; 

      //delete client organization
      foreach($client->getClientPets() as $c){
         if(!in_array($c->getIdEncoded(),$petIds)){
            $c->setIsDeleted(true);
            $em->flush();
         }
      } 
   }

   /**
    * @Route("/report", name="client_report")
    */
   public function report(Request $request, AuthService $authService)
   {
         if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
         if(!$authService->isUserHasAccesses(array('Report Client'))) return $authService->redirectToHome();

         $page_title = ' Client Report'; 
         return $this->render('Client/report.html.twig', [ 
            'page_title' => $page_title,
            'disasterTypes' => $this->getDoctrine()->getManager()->getRepository(DisasterTypeEntity::class)->findBy(['branch' => $authService->getUser()->getBranch(), 'isDeleted' => 0])

         ]
         );
   }

       /**
     * @Route("/report/csv", name = "client_report_csv")
     */
    public function clientReportCsv(Request $request, AuthService $authService){

      ini_set('memory_limit', '2048M');
      set_time_limit(0);

      $data = $request->request->all();
      $em = $this->getDoctrine()->getManager();
      $userData = $this->get('session')->get('userData');
      $disasterType = $em->getRepository(DisasterTypeEntity::class)->find($data['disasterType']);
      $columnRange = range('A', 'S');

      $cellsData [] = ['cell' => 'A1', 'data' =>  'CITY SOCIAL WELFARE AND DEVELOPMENT OFFICE'];
      $cellsData [] = ['cell' => 'A2', 'data' =>  'CITY OF TANAUAN'];
      $cellsData [] = ['cell' => 'A3', 'data' =>  'MASTERLIST OF  TANAUAN CITY INTERNALLY DISPLACED PERONS (IDPs)'];
      $cellsData [] = ['cell' => 'B6', 'data' =>  'REGION'];
      $cellsData [] = ['cell' => 'C6', 'data' =>  'IV-A CALABARZON'];
      $cellsData [] = ['cell' => 'B7', 'data' =>  'TYPE OF DISASTER:'];
      $cellsData [] = ['cell' => 'C7', 'data' =>  $disasterType->getDescription()];
      $cellsData [] = ['cell' => 'B8', 'data' =>  'Date of Occurence:'];
      $cellsData [] = ['cell' => 'C8', 'data' =>  $data['date_occurence']];

      $cellsData [] = ['cell' => 'B9', 'data' =>  'Place of Origin(Barangay)'];
      $cellsData [] = ['cell' => 'C9', 'data' =>  'Head of The Family'];
      $cellsData [] = ['cell' => 'D9', 'data' =>  'Spouse'];
      $cellsData [] = ['cell' => 'E9', 'data' =>  'Number of Individuals'];
      $cellsData [] = ['cell' => 'F9', 'data' =>  'M'];
      $cellsData [] = ['cell' => 'G9', 'data' =>  'F'];
      $cellsData [] = ['cell' => 'H9', 'data' =>  '0-14'];
      $cellsData [] = ['cell' => 'I9', 'data' =>  '15-21'];
      $cellsData [] = ['cell' => 'J9', 'data' =>  '22 And Above'];
      $cellsData [] = ['cell' => 'K9', 'data' =>  'Senior'];
      $cellsData [] = ['cell' => 'L9', 'data' =>  'PWD'];
      $cellsData [] = ['cell' => 'M9', 'data' =>  'Pregnant'];
      $cellsData [] = ['cell' => 'N9', 'data' =>  'Lactating'];

      $clients = $this->getDoctrine()->getManager()->getRepository(ClientEntity::class)->report($data, $this->get('session')->get('userData'));

      $row = 10;
      foreach($clients as $client){
         
         $membersExplode = explode(',', $client['memberNames']);
         $rolesExplode = explode(',', $client['memberRoles']);
         $ageExplode = explode(',', $client['memberAges']); 
         $pwdExplode = explode(',', $client['memberPwds']); 
         $pregnantExplode = explode(',', $client['memberPregnants']); 
         $lactatingExplode = explode(',', $client['memberLactatings']);
         $genderExplode = explode(',', $client['memberGenders']);

         $pwdCtr = 0;
         $pregCtr = 0;
         $lacCtr = 0;
         $mCtr = 0;
         $fCtr =0;
         $under14Ctr =0;
         $under15To21Ctr =0;
         $above22Ctr =0;
         $headOfFamily = '';
         $spouse = '';
         $seniorCtr = 0;
         foreach($rolesExplode as $k => $c){

            if($c == 'Head Of The Family'){ $headOfFamily = $membersExplode[$k]; }
            if($c == 'Spouse'){ $spouse = $membersExplode[$k]; }
            if($ageExplode[$k] <= 14){ $under14Ctr++; }
            if($ageExplode[$k] >= 15 && $ageExplode[$k] <= 21 ){ $under15To21Ctr++; }
            if($ageExplode[$k] >= 22  ){ $above22Ctr++; }
            if($genderExplode[$k] == 'Male'){ $mCtr++; }
            if($genderExplode[$k] == 'Female'){ $fCtr++; }
            if($pwdExplode[$k]){ $pwdCtr++; }
            if($pregnantExplode[$k]){ $pregCtr++; }
            if($lactatingExplode[$k]){ $lacCtr++; }
            if($ageExplode[$k] >= 60){ $seniorCtr++; }
         }

         $cellsData [] = ['cell' => 'B' . $row, 'data' =>  $client['barangay']];
         $cellsData [] = ['cell' => 'C' . $row, 'data' =>  $headOfFamily];
         $cellsData [] = ['cell' => 'D' . $row, 'data' =>  $spouse];
         $cellsData [] = ['cell' => 'E' . $row, 'data' =>  count($membersExplode)];
         $cellsData [] = ['cell' => 'F' . $row, 'data' =>  $mCtr];
         $cellsData [] = ['cell' => 'G' . $row, 'data' =>  $fCtr];
         $cellsData [] = ['cell' => 'H' . $row, 'data' =>  $under14Ctr];
         $cellsData [] = ['cell' => 'I' . $row, 'data' =>  $under15To21Ctr];
         $cellsData [] = ['cell' => 'J' . $row, 'data' =>  $above22Ctr];
         $cellsData [] = ['cell' => 'K' . $row, 'data' =>  $seniorCtr];
         $cellsData [] = ['cell' => 'L' . $row, 'data' =>  $pwdCtr];
         $cellsData [] = ['cell' => 'M' . $row, 'data' =>  $pregCtr];
         $cellsData [] = ['cell' => 'N' . $row, 'data' =>  $lacCtr];
     
      }

      $page_title =  strtolower('client-report');
      return $this->export_to_excel($columnRange, $cellsData, $page_title);
  }

   private function export_to_excel($columnRange, $cellsData, $page_title, $customStyle=array()) {


      $spreadSheet = new SpreadSheet();
      $activeSheet = $spreadSheet->getActiveSheet(0);

      foreach($cellsData as $cellData) {
          $activeSheet->getCell($cellData['cell'])->setValue($cellData['data']);
      }

      $activeSheet->getColumnDimension('A')->setAutoSize(true);

      $writer = new Xlsx($spreadSheet);

      $response =  new StreamedResponse(
          function () use ($writer) {
              $writer->save('php://output');
          }
      );

      $response->headers->set('Content-Type', 'text/vnd.ms-excel; charset=utf-8');
      $response->headers->set('Content-Disposition', 'attachment;filename="' . $page_title . '.xlsx"');
      $response->headers->set('Pragma', 'public');
      $response->headers->set('Cache-Control', 'maxage=1');

      return $response;
  }


}
