<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;

use App\Service\AuthService;
use App\Entity\ClientPetEntity;
use App\Entity\BranchEntity;




use App\Form\ClientPetForm;

/**
 * @Route("/client_pet")
 */
class ClientPetController extends AbstractController
{

      /**
    * @Route("", name="client_pet_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Client Pet'))) return $authService->redirectToHome();
      
      $page_title = ' Pet'; 
      return $this->render('ClientPet/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/client_pet/index.js') ]
      );
   }

       /**
     * @Route("/ajax_detail", name="client_ajax_detail")
     */
    public function ajaxDetail(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
        $em = $this->getDoctrine()->getManager();
        $id = $request->query->get('id');
        $clientPet = $em->getRepository(ClientPetEntity::class)->find(base64_decode($id)); 
        

        if(!$clientPet){

            $result['success'] = false;
            $result['msg'] = 'Opps something went wrong please try again later';
        } else {

       
            $result['html'] = $this->renderView('ClientPet/ajax_detail.html.twig', [
                  'page_title' => 'Pet Details',
                  'clientPet' => $clientPet

            ]);
        }
      
       return new JsonResponse($result);
    }

       /**
    * @Route("/ajax_list", name="client_pet_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(ClientPetEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

   
    /**
    * @Route("/autocomplete_pet", name="client_pet_autocomplete")
    */
    public function petAutocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'clientPets',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(ClientPetEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }


}
