<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\BranchRepository")
 * @ORM\Table(name="branch")
 * @ORM\HasLifecycleCallbacks()
 */

class BranchEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="code", type="string")
     */
    protected $code;
    
    /**
     * @ORM\Column(name="description", type="string")
     */
    protected $description;

    /**
     * @ORM\ManyToOne(targetEntity="CompanyEntity", inversedBy="branches")
     * @ORM\JoinColumn(name="company_id", referencedColumnName="id", nullable=true)
     */
    protected $company;

    /**
     * @ORM\OneToMany(targetEntity="UserEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $users;

    /**
     * @ORM\OneToMany(targetEntity="BranchVariableEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $branchVariables;

     /**
     * @ORM\OneToMany(targetEntity="BreedEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $breeds;

     /**
     * @ORM\OneToMany(targetEntity="RoomTypeEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $roomTypes;

     /**
     * @ORM\OneToMany(targetEntity="SpeciesEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $speciess;
    
     /**
     * @ORM\OneToMany(targetEntity="RoomEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $rooms;

     /**
     * @ORM\OneToMany(targetEntity="ServiceEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $services;

    /**
     * @ORM\OneToMany(targetEntity="LodgeEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $lodges;

    /**
     * @ORM\OneToMany(targetEntity="GroomingEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $groomings;

    /**
     * @ORM\OneToMany(targetEntity="PaymentTypeEntity", mappedBy="branch", cascade={"remove"})
     */
    protected $paymentTypes;


    
    public function __construct($data = null)
    {
        $this->users = new ArrayCollection();
        $this->branchVariables = new ArrayCollection();
        $this->breeds = new ArrayCollection();
        $this->roomTypes = new ArrayCollection();
        $this->speciess = new ArrayCollection();
        $this->rooms = new ArrayCollection();
        $this->services = new ArrayCollection();
        $this->lodges = new ArrayCollection();
        $this->groomings = new ArrayCollection();
        $this->paymentTypes = new ArrayCollection();
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					Branch Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

/**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return BranchEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }
    /*--------------------------------------------------------------------------------------------------------*/
    /*					    Defined Setters and Getters													      */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getCompany(): ?CompanyEntity
    {
        return $this->company;
    }

    public function setCompany(?CompanyEntity $company): self
    {
        $this->company = $company;

        return $this;
    }

    /**
     * @return Collection<int, UserEntity>
     */
    public function getUsers(): Collection
    {
        return $this->users;
    }

    public function addUser(UserEntity $user): self
    {
        if (!$this->users->contains($user)) {
            $this->users[] = $user;
            $user->setBranch($this);
        }

        return $this;
    }

    public function removeUser(UserEntity $user): self
    {
        if ($this->users->removeElement($user)) {
            // set the owning side to null (unless already changed)
            if ($user->getBranch() === $this) {
                $user->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, BranchVariableEntity>
     */
    public function getBranchVariables(): Collection
    {
        return $this->branchVariables;
    }

    public function addBranchVariable(BranchVariableEntity $branchVariable): self
    {
        if (!$this->branchVariables->contains($branchVariable)) {
            $this->branchVariables[] = $branchVariable;
            $branchVariable->setBranch($this);
        }

        return $this;
    }

    public function removeBranchVariable(BranchVariableEntity $branchVariable): self
    {
        if ($this->branchVariables->removeElement($branchVariable)) {
            // set the owning side to null (unless already changed)
            if ($branchVariable->getBranch() === $this) {
                $branchVariable->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, BreedEntity>
     */
    public function getBreeds(): Collection
    {
        return $this->breeds;
    }

    public function addBreed(BreedEntity $breed): self
    {
        if (!$this->breeds->contains($breed)) {
            $this->breeds[] = $breed;
            $breed->setBranch($this);
        }

        return $this;
    }

    public function removeBreed(BreedEntity $breed): self
    {
        if ($this->breeds->removeElement($breed)) {
            // set the owning side to null (unless already changed)
            if ($breed->getBranch() === $this) {
                $breed->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, RoomTypeEntity>
     */
    public function getRoomTypes(): Collection
    {
        return $this->roomTypes;
    }

    public function addRoomType(RoomTypeEntity $roomType): self
    {
        if (!$this->roomTypes->contains($roomType)) {
            $this->roomTypes[] = $roomType;
            $roomType->setBranch($this);
        }

        return $this;
    }

    public function removeRoomType(RoomTypeEntity $roomType): self
    {
        if ($this->roomTypes->removeElement($roomType)) {
            // set the owning side to null (unless already changed)
            if ($roomType->getBranch() === $this) {
                $roomType->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, SpeciesEntity>
     */
    public function getSpeciess(): Collection
    {
        return $this->speciess;
    }

    public function addSpeciess(SpeciesEntity $speciess): self
    {
        if (!$this->speciess->contains($speciess)) {
            $this->speciess[] = $speciess;
            $speciess->setBranch($this);
        }

        return $this;
    }

    public function removeSpeciess(SpeciesEntity $speciess): self
    {
        if ($this->speciess->removeElement($speciess)) {
            // set the owning side to null (unless already changed)
            if ($speciess->getBranch() === $this) {
                $speciess->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, RoomEntity>
     */
    public function getRooms(): Collection
    {
        return $this->rooms;
    }

    public function addRoom(RoomEntity $room): self
    {
        if (!$this->rooms->contains($room)) {
            $this->rooms[] = $room;
            $room->setBranch($this);
        }

        return $this;
    }

    public function removeRoom(RoomEntity $room): self
    {
        if ($this->rooms->removeElement($room)) {
            // set the owning side to null (unless already changed)
            if ($room->getBranch() === $this) {
                $room->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, ServiceEntity>
     */
    public function getServices(): Collection
    {
        return $this->services;
    }

    public function addService(ServiceEntity $service): self
    {
        if (!$this->services->contains($service)) {
            $this->services[] = $service;
            $service->setBranch($this);
        }

        return $this;
    }

    public function removeService(ServiceEntity $service): self
    {
        if ($this->services->removeElement($service)) {
            // set the owning side to null (unless already changed)
            if ($service->getBranch() === $this) {
                $service->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, LodgeEntity>
     */
    public function getLodges(): Collection
    {
        return $this->lodges;
    }

    public function addLodge(LodgeEntity $lodge): self
    {
        if (!$this->lodges->contains($lodge)) {
            $this->lodges[] = $lodge;
            $lodge->setBranch($this);
        }

        return $this;
    }

    public function removeLodge(LodgeEntity $lodge): self
    {
        if ($this->lodges->removeElement($lodge)) {
            // set the owning side to null (unless already changed)
            if ($lodge->getBranch() === $this) {
                $lodge->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, GroomingEntity>
     */
    public function getGroomings(): Collection
    {
        return $this->groomings;
    }

    public function addGrooming(GroomingEntity $grooming): self
    {
        if (!$this->groomings->contains($grooming)) {
            $this->groomings[] = $grooming;
            $grooming->setBranch($this);
        }

        return $this;
    }

    public function removeGrooming(GroomingEntity $grooming): self
    {
        if ($this->groomings->removeElement($grooming)) {
            // set the owning side to null (unless already changed)
            if ($grooming->getBranch() === $this) {
                $grooming->setBranch(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, PaymentTypeEntity>
     */
    public function getPaymentTypes(): Collection
    {
        return $this->paymentTypes;
    }

    public function addPaymentType(PaymentTypeEntity $paymentType): self
    {
        if (!$this->paymentTypes->contains($paymentType)) {
            $this->paymentTypes[] = $paymentType;
            $paymentType->setBranch($this);
        }

        return $this;
    }

    public function removePaymentType(PaymentTypeEntity $paymentType): self
    {
        if ($this->paymentTypes->removeElement($paymentType)) {
            // set the owning side to null (unless already changed)
            if ($paymentType->getBranch() === $this) {
                $paymentType->setBranch(null);
            }
        }

        return $this;
    }

   
   
 
}
