<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\LodgeServiceRepository")
 * @ORM\Table(name="lodge_service")
 * @ORM\HasLifecycleCallbacks()
 */

class LodgeServiceEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="status", type="string")
     */
    protected $status;

    /**
     * @ORM\Column(name="service_rate", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $serviceRate;
    
    /**
     * @ORM\ManyToOne(targetEntity="LodgeEntity", inversedBy="lodgeServices")
     * @ORM\JoinColumn(name="lodge_id", referencedColumnName="id", nullable=true)
     */
    protected $lodge;

    /**
     * @ORM\ManyToOne(targetEntity="ServiceEntity", inversedBy="lodgeServices")
     * @ORM\JoinColumn(name="service_id", referencedColumnName="id", nullable=true)
     */
    protected $service;

    /**
     * @ORM\ManyToOne(targetEntity="UserEntity", inversedBy="lodgeServices")
     * @ORM\JoinColumn(name="user_id", referencedColumnName="id", nullable=true)
     */
    protected $user;

           /**
     * @ORM\OneToMany(targetEntity="BranchSmsEntity", mappedBy="lodgeService", cascade={"remove"})
     */
    protected $branchSmss;

    public function __construct($data = null)
    {
        $this->branchSmss = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return LodgeServiceEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getLodge(): ?LodgeEntity
    {
        return $this->lodge;
    }

    public function setLodge(?LodgeEntity $lodge): self
    {
        $this->lodge = $lodge;

        return $this;
    }

    public function getServiceRate(): ?string
    {
        return $this->serviceRate;
    }

    public function setServiceRate(?string $serviceRate): self
    {
        $this->serviceRate = str_replace(',','', $serviceRate);

        return $this;
    }

    public function getService(): ?ServiceEntity
    {
        return $this->service;
    }

    public function setService(?ServiceEntity $service): self
    {
        $this->service = $service;

        return $this;
    }

    public function getUser(): ?UserEntity
    {
        return $this->user;
    }

    public function setUser(?UserEntity $user): self
    {
        $this->user = $user;

        return $this;
    }

    /**
     * @return Collection<int, BranchSmsEntity>
     */
    public function getBranchSmss(): Collection
    {
        return $this->branchSmss;
    }

    public function addBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if (!$this->branchSmss->contains($branchSmss)) {
            $this->branchSmss[] = $branchSmss;
            $branchSmss->setLodgeService($this);
        }

        return $this;
    }

    public function removeBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if ($this->branchSmss->removeElement($branchSmss)) {
            // set the owning side to null (unless already changed)
            if ($branchSmss->getLodgeService() === $this) {
                $branchSmss->setLodgeService(null);
            }
        }

        return $this;
    }

  
   
}

