<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;


use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;

use App\Form\DataTransformer\DataTransformer;
use App\Form\DataTransformer\TimeTransformer;

use App\Entity\LodgeEntity;
use App\Entity\RoomEntity;
use App\Entity\BranchEntity;
use App\Entity\ClientPetEntity;


class LodgeForm extends AbstractType
{
   
    private $manager;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)  
            ->add('is_ready_for_pick_up', CheckboxType::class, [
                'label'    => 'Ready for pickup',
                'required' => false,
                'label_attr' => ['class' => 'form-check-label']
            ])
            ->add('branch', HiddenType::class, array('data' => $options['branchId']))
            ->add('room', HiddenType::class, array('data' => $options['roomId']))
            ->add('client_pet', HiddenType::class)
            
  
            ->addEventListener(FormEvents::PRE_SET_DATA, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();
                $cp = $data->getClientPet();

                $form
                    ->add('client_pet_desc', TextType::class, array(
                        'label' => 'Client',
                        'label_attr' => array(
                            'class' => 'middle required  col-form-label'
                        ),
                        'required' => true,
                        'attr' => array(                            
                            'class' => 'form-control ',
                        ),
                        'mapped' => false,
                        'data' => $cp ? $cp->getClient()->getFullName() . ' - ' . $cp->getPetName() : ''
                    ))
                    ->add('date_time_in', TextType::class, array(
                        'label' => 'Date Start',
                        'required' => true,
                        'label_attr' => array(
                            'class' => 'middle required'
                        ),
                        'attr' => [ 'class' => 'form-control required datepicker'],
                        'mapped' => false,
                        'data' => $data->getTimeIn() ? $data->getTimeIn()->format('m/d/Y')  : date('m/d/Y')

                    ))
                    
                    ->add('date_time_out', TextType::class, array(
                        'label' => 'Date End',
                        'required' => false,
                        'label_attr' => array(
                            'class' => 'middle'
                        ),
                        'attr' => [ 'class' => 'form-control datepicker'],
                        'mapped' => false,
                        'data' => $data->getTimeOut() ? $data->getTimeOut()->format('m/d/Y')  : ''
                        
                    ))
                    ->add('time_in', TextType::class, array(
                        'label' => 'Time In',
                        'required' => true,
                        'label_attr' => array(
                            'class' => 'middle '
                        ),
                        'attr' => [ 'class' => 'form-control required timepicker'],
                        'data' => $data->getTimeIn() ? $data->getTimeIn()->format('h:i A')  : date('h:i A')


                        
                    )) 
                    ->add('time_out', TextType::class, array(
                        'label' => 'Time Out',
                        'label_attr' => array(
                            'class' => 'middle '
                        ),
                        'attr' => [ 'class' => 'form-control  timepicker'],
                        'data' => $data->getTimeOut() ? $data->getTimeOut()->format('h:i A')  : "",
                        'required' => false

                    ));



                 
            })
            ->addEventListener(FormEvents::PRE_SUBMIT, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();

                $timeInDate = $data['date_time_in'];
                $data['time_in'] = !empty($timeInDate) && !empty($data['time_in']) ?  new \Datetime($timeInDate . ' ' . $data['time_in']) : null;
                
                if(isset($data['date_time_in']) && isset($data['time_out'])){
                    $timeOutDate = $data['date_time_out'];
                    $data['time_out'] = !empty($timeOutDate) && !empty($data['time_out']) ?  new \Datetime($timeOutDate . ' ' . $data['time_out']) : null;

                }
                
                
                $event->setData($data);
                
                 
            })

            ->addEventListener(FormEvents::POST_SUBMIT, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();
                $room = $form->getData()->getRoom();
                
                if(is_null($data->getTimeOut())){
                    $data->setTimeOut(new \Datetime(date("Y-m-d H:i",strtotime("+".$room->getMinimumHour()." hour",strtotime(date('Y-m-d H:i'))))));
               
                } 
                
                 
            });
            $builder->get('branch')->addModelTransformer(new DataTransformer($this->manager, BranchEntity::class, true, $options['branchId']));
            $builder->get('room')->addModelTransformer(new DataTransformer($this->manager, RoomEntity::class, true, $options['roomId']));
            $builder->get('client_pet')->addModelTransformer(new DataTransformer($this->manager, ClientPetEntity::class, false));
            // $builder->get('time_in')->addModelTransformer(new TimeTransformer());
            // $builder->get('time_out')->addModelTransformer(new TimeTransformer());
    }

    public function getName()
    {
        return 'lodge';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\LodgeEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'branchId'    => null,
            'roomId'    => null,
            'allow_extra_fields' => true,
        ));
    }
}