<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;


use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;

use App\Form\DataTransformer\DataTransformer;

use App\Form\DataTransformer\DatetimeTransformer;

use App\Entity\LodgeEntity;
use App\Entity\LodgePaymentEntity;
use App\Entity\PaymentTypeEntity;



class LodgePaymentForm extends AbstractType
{
   
    private $manager;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)  
            ->add('lodge', HiddenType::class, array('data' => $options['lodgeId']))
            ->add('payment_type', HiddenType::class)
            ->add('payment_date', TextType::class, array(
                'label' => 'Date ',
                'required' => true,
                'label_attr' => array(
                    'class' => 'middle required'
                ),
                'attr' => [ 'class' => 'form-control required datepicker']
            ))
            ->add('amount', TextType::class, array(
                'label' => 'Amount',
                'required' => true,
                'label_attr' => array(
                    'class' => 'middle required'
                ),
                'attr' => [ 'class' => 'form-control required amt']
            ))
  
            ->addEventListener(FormEvents::PRE_SET_DATA, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();
                $pt = $data->getPaymentType();

                $form
                    ->add('payment_type_desc', TextType::class, array(
                        'label' => 'Payment Type',
                        'label_attr' => array(
                            'class' => 'middle required  col-form-label'
                        ),
                        'required' => true,
                        'attr' => array(                            
                            'class' => 'form-control ',
                        ),
                        'mapped' => false,
                        'data' => $pt ? $pt->getDescription() . ' - ' . $pt->getDescription() : ''
                    ));
                 
            });  
            
            $builder->get('lodge')->addModelTransformer(new DataTransformer($this->manager, LodgeEntity::class, true, $options['lodgeId']));
            $builder->get('payment_type')->addModelTransformer(new DataTransformer($this->manager, PaymentTypeEntity::class, false));
            $builder->get('payment_date')->addModelTransformer(new DatetimeTransformer());

  
    }

    public function getName()
    {
        return 'lodgePayment';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\LodgePaymentEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'lodgeId'    => null
        ));

    }
}